/*********************************************************************/
/*                 Initialisation partielle des USARTs               */
/*                         de l'ATmega164                            */
/*********************************************************************/
// ATmega164P le 27 11 08
#include "ATmega164_VMG.h"
#pragma interrupt_handler     IT_Rx0:iv_USART0_RX

extern void Push_Tampon (char);
extern byte Etat_Tampon;

// Initialisation des USARTs avec quartz de 7.3728 MHz 
// Mode asynchrone, Pas de division de Xtal
// ----------------------------------------------------
//   baud      UBRR0 (U2X0 = 1)
//   2400      383
//   4800      191
//   9600       95
//  14400       63
//  19200       47
//  28800       31
//  38400       23
//  57600       15
//  76800       11
// 115200        7
// 230400        3
// long_mot : 5, 6, 7, 8 ou 9 bits
// parite = N, E ou O; sans parite, parite paire, parite impaire
// nb_stop = 1 ou 2;

// Configuration USART0
void Init_USART0 (long int baud, char long_mot, char parite, char nb_stop)
     {
     unsigned char prop;          
     UCSR0A = (1<<U2X0);
     prop = (char)(baud/2400);
// Format de transmission
     switch (long_mot)
          {
          case 6:   UCSR0C |= (1<<UCSZ00);
                    break;
          case 7:   UCSR0C |= (1<<UCSZ01);
                    break;
          case 8:   UCSR0C |= (1<<UCSZ00) | (1<<UCSZ01);
                    break;
          case 9:   UCSR0C |= (1<<UCSZ00) | (1<<UCSZ01);
                    UCSR0B  = (1<<UCSZ02);
                    break;
          }
     switch (parite)
          {
          case 'E': UCSR0C |= (1<<UPM01);
                    break;
          case 'O': UCSR0C |= (1<<UPM00) | (1<<UPM01);
                    break;
          case 'e': UCSR0C |= (1<<UPM01);
                    break;
          case 'o': UCSR0C |= (1<<UPM00) | (1<<UPM01);
                    break;
          }
     if (nb_stop == 2)   UCSR0C |= (1<<USBS0);
// Vitesse de transmission     
     switch (prop)
          {
          case 1:   UBRR0L = 127;       // 2400
                    UBRR0H = 1;
                    break; 
          case 2:   UBRR0L = 191;       // 4800
                    break; 
          case 4:   UBRR0L = 95;        // 9600
                    break; 
          case 6:   UBRR0L = 63;        // 14400
                    break; 
          case 8:   UBRR0L = 47;        // 19200
                    break; 
          case 12:  UBRR0L = 31;        // 28800
                    break; 
          case 16:  UBRR0L = 23;        // 38400
                    break;
          case 24:  UBRR0L = 15;        // 57600
                    break; 
          case 32:  UBRR0L = 11;        // 76800
                    break; 
          case 48:  UBRR0L = 7;         // 115200
                    break; 
          case 96:  UBRR0L = 3;         // 230400
                    break;
          default:  UBRR0L = 95;        // 9600 Bd par defaut
          }
     }                                                            
 
// Configuration USART1
void Init_USART1 (long int baud, char long_mot, char parite, char nb_stop)
     {
     unsigned char prop;          
     UCSR1A = (1<<U2X1);
     prop = (char)(baud/2400);
// Format de transmission
     switch (long_mot)
          {
          case 6:   UCSR1C |= (1<<UCSZ10);
                    break;
          case 7:   UCSR1C |= (1<<UCSZ11);
                    break;
          case 8:   UCSR1C |= (1<<UCSZ10) | (1<<UCSZ11);
                    break;
          case 9:   UCSR1C |= (1<<UCSZ10) | (1<<UCSZ11);
                    UCSR1B  = (1<<UCSZ12);
                    break;
          }
     switch (parite)
          {
          case 'E': UCSR1C |= (1<<UPM11);
                    break;
          case 'O': UCSR1C |= (1<<UPM10) | (1<<UPM11);
                    break;
          case 'e': UCSR1C |= (1<<UPM11);
                    break;
          case 'o': UCSR1C |= (1<<UPM10) | (1<<UPM11);
                    break;
          }
     if (nb_stop == 2)   UCSR1C |= (1<<USBS1);
// Vitesse de transmission     
     switch (prop)
          {
          case 1:   UBRR1L = 127;       // 2400
                    UBRR1H = 1;
                    break; 
          case 2:   UBRR1L = 191;       // 4800
                    break; 
          case 4:   UBRR1L = 95;        // 9600
                    break; 
          case 6:   UBRR1L = 63;        // 14400
                    break; 
          case 8:   UBRR1L = 47;        // 19200
                    break; 
          case 12:  UBRR1L = 31;        // 28800
                    break; 
          case 16:  UBRR1L = 23;        // 38400
                    break;
          case 24:  UBRR1L = 15;        // 57600
                    break; 
          case 32:  UBRR1L = 11;        // 76800
                    break; 
          case 48:  UBRR1L = 7;         // 115200
                    break; 
          case 96:  UBRR1L = 3;         // 230400
                    break;
          default:  UBRR1L = 95;        // 9600 Bd par defaut
          }
     }                                                            
 
// Transmission d'un octet sur USART1
void Tx1_Data (unsigned char data)
	{
	while (!(UCSR1A & (1<<UDRE1)));      // Attente emission octet prcdent
	UDR1 = data;
	}

// IT de rception sur USART0
void IT_Rx0 (void)
	{
	if (Etat_Tampon != FULL)
		Push_Tampon (UDR0);
	}



